package com.yizhi.live.application.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.plugins.Page;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.live.application.domain.LiveActivity;
import com.yizhi.live.application.request.LiveReleaseReviewReq;
import com.yizhi.live.application.service.ILiveActivityService;
import com.yizhi.live.application.service.ILiveReplayService;
import com.yizhi.live.application.service.ILiveTokenService;
import com.yizhi.live.application.service.IScopeAuthorizationService;
import com.yizhi.live.application.vo.*;
import com.yizhi.util.application.domain.BizResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

@Api(tags = "直播接口")
@RestController
@RequestMapping("/remote/live")
public class RemoteLiveController {

    private static final Logger LOG = LoggerFactory.getLogger(RemoteLiveController.class);

    @Autowired
    ILiveActivityService liveActivityService;

    @Autowired
    IdGenerator idGenerator;

    @Autowired
    ILiveTokenService liveTokenService;

    @Autowired
    IScopeAuthorizationService scopeAuthorizationService;
    @Autowired
    ILiveReplayService replayService;


    /**
     * 查询近7天的已完成的直播（包含今天的）
     *
     * @return
     */
    @GetMapping("/sevenDays/get")
    public List<LiveActivity> findWithinSevenDaysLive() {
        RequestContext rt = ContextHolder.get();
        return liveActivityService.findWithinSevenDaysLive(rt.getCompanyId(), rt.getSiteId(), rt.getAccountId());
    }


    @GetMapping("/statistics/online/count/realtime")
    @ApiOperation(value = "实时查询直播在线人数;多个频道用逗号隔开")
    public List<LiveOnlineCountVo> liveCountOnlineRealTime(@RequestParam("channelIds") String channelIds) {
        try {
            return liveActivityService.getChannelOnlineCountRealTime(channelIds);
        } catch (JsonProcessingException e) {
            LOG.error("获取频道在线人数异常",e);
        }
        return new ArrayList<>();
    }

    /**
     * 保利威回调接口  直播状态修改
     * @param channelId 频道id
     * @param status 直播状态 ：live表示开始直播，end表示直播结束
     */
    @GetMapping("/status/callback")
    @ApiOperation(value = "直播状态改变,保利威回调接口")
    public BizResponse liveStatusCallback(@RequestParam(value = "channelId") String channelId
            ,@RequestParam(value = "status") String status
            ,@RequestParam(value = "timestamp") Long timestamp
            ,@RequestParam(value = "sign") String sign
            ,@RequestParam(value = "sessionId") String sessionId
            ,@RequestParam(value = "startTime") String startTime
            ,@RequestParam(value = "endTime") String endTime) {

        LOG.info("保利威直播状态回调；channelId={},status={},sign={},startTime={},endTime={},sessionId={}",channelId,status,sign,startTime,endTime,sessionId);
        return liveActivityService.liveStatusCallback(channelId,status,timestamp,sign,sessionId,startTime,endTime);
    }

    @GetMapping("/transfer/callback")
    @ApiOperation(value = "直播转存成功,保利威回调接口")
    public BizResponse<String> liveTransferCallback(@RequestBody PolyvLiveBackCallbackVo liveBackCallbackVo) {
        LOG.info("直播转存成功,保利威回调参数channelId = {};",liveBackCallbackVo.getChannelId());
        return liveActivityService.liveTransferCallback(liveBackCallbackVo);
    }

    @GetMapping("/play/review/list")
    @ApiOperation(value = "查询所有直播回放列表 废弃")
    @Deprecated
    public Page<LiveReplayVo> playReviewList(@RequestBody LiveReviewListVo liveReviewListVo) {
        return null;
    }

    @GetMapping("/review/switch/on/live/list")
    @ApiOperation(value = "查询有直播回放的直播间列表  废弃！！")
    @Deprecated
    public BizResponse<List<LiveTitleVo>> reviewLiveList(@RequestBody LiveReviewsVo liveReviewsVo) {
        return liveActivityService.reviewLiveList(liveReviewsVo);
    }

    @GetMapping("/review/switch/on/live/page/list")
    @ApiOperation(value = "分页查询有直播回放的直播间列表")
    public Page<LiveTitleVo> getHasReplayLiveList(@RequestBody LiveReleaseReviewReq replayVo) {
        return liveActivityService.getHasReplayLiveList(replayVo);
    }

    @GetMapping("/reviews/get")
    @ApiOperation(value = "根据直播间查询上架的直播回放")
    public BizResponse<List<LiveReplayVo>> reviewsList(@RequestBody LiveReviewsVo liveReviewsVo) {

        return liveActivityService.reviewsList(liveReviewsVo);
    }

    @GetMapping("/review/one/get")
    @ApiOperation(value = "查询直播回放详情")
    public BizResponse<LiveReplayVo> getReview(@RequestParam(name = "replayId") Long replayId) {
        return replayService.getReplay(replayId);
    }

    @PostMapping("/release/replay/list")
    @ApiOperation(value = "查询所有上架的直播回放列表")
    public Page<LiveReplayVo> getReleaseReplayList(@RequestBody LiveReleaseReviewReq reviewReq) {
        return replayService.getReleaseReplayList(reviewReq);
    }

    @PostMapping("/replay/list")
    @ApiOperation(value = "管理端根据直播间分页查询所有直播回放")
    public Page<LiveReplayVo> getManageReplayList(@RequestBody LiveReviewsVo liveReviewsVo) {

        Page<LiveReplayVo> page = liveActivityService.getReplayList(liveReviewsVo);
        if (null == page){
            return new Page<>(liveReviewsVo.getPageNo(),liveReviewsVo.getPageSize());
        }
        return page;
    }

    @PostMapping("/replay/status/update")
    @ApiOperation(value = "直播回放上架、下架")
    public Boolean updateReplayStatus(@RequestBody LiveReplayOperationVo liveReplayOperationVo) {

        return liveActivityService.updateReplayStatus(liveReplayOperationVo.getReplayId(),liveReplayOperationVo.getOperation());
    }

    @PostMapping("/replay/top/update")
    @ApiOperation(value = "直播回放置顶")
    public Boolean updateReplayTop(@RequestBody LiveReplayOperationVo liveReplayOperationVo) {

        return liveActivityService.updateReplayTop(liveReplayOperationVo.getReplayId(),liveReplayOperationVo.getOperation());
    }


    @PostMapping("/replay/info/update")
    @ApiOperation(value = "直播回放信息修改")
    public Boolean updateReplayInfo(@RequestBody LiveReplayInfoVo infoVo ) {

        return liveActivityService.updateReplayInfos(infoVo);
    }



    

}